@@ -21,6 +21,8 @@ use types::WorkspaceFoldersServerCapabilities;
 use self::schedule::event_loop_thread;
 use self::schedule::Scheduler;
 use self::schedule::Task;
+use crate::session::AllSettings;
+use crate::session::ClientSettings;
 use crate::session::Session;
 use crate::PositionEncoding;
 
@@ -28,6 +30,8 @@ mod api;
 mod client;
 mod schedule;
 
+pub(crate) use client::ClientSender;
+
 pub(crate) type Result<T> = std::result::Result<T, api::Error>;
 
 pub struct Server {
@@ -42,20 +46,41 @@ impl Server {
     pub fn new(worker_threads: NonZeroUsize) -> crate::Result<Self> {
         let (conn, threads) = lsp::Connection::stdio();
 
+        crate::message::init_messenger(&conn.sender);
+
         let (id, params) = conn.initialize_start()?;
 
         let init_params: types::InitializeParams = serde_json::from_value(params)?;
 
         let client_capabilities = init_params.capabilities;
-        let server_capabilities = Self::server_capabilities(&client_capabilities);
+        let position_encoding = Self::find_best_position_encoding(&client_capabilities);
+        let server_capabilities = Self::server_capabilities(position_encoding);
+
+        let AllSettings {
+            global_settings,
+            mut workspace_settings,
+        } = AllSettings::from_value(init_params.initialization_options.unwrap_or_default());
+
+        let mut workspace_for_uri = |uri| {
+            let Some(workspace_settings) = workspace_settings.as_mut() else {
+                return (uri, ClientSettings::default());
+            };
+            let settings = workspace_settings.remove(&uri).unwrap_or_else(|| {
+                tracing::warn!("No workspace settings found for {uri}");
+                ClientSettings::default()
+            });
+            (uri, settings)
+        };
 
         let workspaces = init_params
             .workspace_folders
-            .map(|folders| folders.into_iter().map(|folder| folder.uri).collect())
-            .or_else(|| init_params.root_uri.map(|u| vec![u]))
+            .map(|folders| folders.into_iter().map(|folder| {
+                workspace_for_uri(folder.uri)
+            }).collect())
             .or_else(|| {
-                tracing::debug!("No root URI or workspace(s) were provided during initialization. Using the current working directory as a default workspace...");
-                Some(vec![types::Url::from_file_path(std::env::current_dir().ok()?).ok()?])
+                tracing::debug!("No workspace(s) were provided during initialization. Using the current working directory as a default workspace...");
+                let uri = types::Url::from_file_path(std::env::current_dir().ok()?).ok()?;
+                Some(vec![workspace_for_uri(uri)])
             })
             .ok_or_else(|| {
                 anyhow::anyhow!("Failed to get the current working directory while creating a default workspace.")
@@ -73,10 +98,15 @@ impl Server {
 
         Ok(Self {
             conn,
-            client_capabilities,
             threads,
             worker_threads,
-            session: Session::new(&server_capabilities, &workspaces)?,
+            session: Session::new(
+                &client_capabilities,
+                position_encoding,
+                global_settings,
+                workspaces,
+            )?,
+            client_capabilities,
         })
     }
 
@@ -177,8 +207,8 @@ impl Server {
         }
     }
 
-    fn server_capabilities(client_capabilities: &ClientCapabilities) -> types::ServerCapabilities {
-        let position_encoding = client_capabilities
+    fn find_best_position_encoding(client_capabilities: &ClientCapabilities) -> PositionEncoding {
+        client_capabilities
             .general
             .as_ref()
             .and_then(|general_capabilities| general_capabilities.position_encodings.as_ref())
@@ -188,19 +218,23 @@ impl Server {
                     .filter_map(|encoding| PositionEncoding::try_from(encoding).ok())
                     .max() // this selects the highest priority position encoding
             })
-            .unwrap_or_default();
+            .unwrap_or_default()
+    }
+
+    fn server_capabilities(position_encoding: PositionEncoding) -> types::ServerCapabilities {
         types::ServerCapabilities {
             position_encoding: Some(position_encoding.into()),
             code_action_provider: Some(types::CodeActionProviderCapability::Options(
                 CodeActionOptions {
-                    code_action_kinds: Some(vec![
-                        CodeActionKind::QUICKFIX,
-                        CodeActionKind::SOURCE_ORGANIZE_IMPORTS,
-                    ]),
+                    code_action_kinds: Some(
+                        SupportedCodeAction::all()
+                            .map(SupportedCodeAction::to_kind)
+                            .collect(),
+                    ),
                     work_done_progress_options: WorkDoneProgressOptions {
                         work_done_progress: Some(true),
                     },
-                    resolve_provider: Some(false),
+                    resolve_provider: Some(true),
                 },
             )),
             workspace: Some(types::WorkspaceServerCapabilities {
@@ -236,3 +270,46 @@ impl Server {
         }
     }
 }
+
+/// The code actions we support.
+#[derive(Clone, Copy, Debug, PartialEq, Eq, Hash)]
+pub(crate) enum SupportedCodeAction {
+    /// Maps to the `quickfix` code action kind. Quick fix code actions are shown under
+    /// their respective diagnostics. Quick fixes are only created where the fix applicability is
+    /// at least [`ruff_diagnostics::Applicability::Unsafe`].
+    QuickFix,
+    /// Maps to the `source.fixAll` and `source.fixAll.ruff` code action kinds.
+    /// This is a source action that applies all safe fixes to the currently open document.
+    SourceFixAll,
+    /// Maps to `source.organizeImports` and `source.organizeImports.ruff` code action kinds.
+    /// This is a source action that applies import sorting fixes to the currently open document.
+    #[allow(dead_code)] // TODO: remove
+    SourceOrganizeImports,
+}
+
+impl SupportedCodeAction {
+    /// Returns the LSP code action kind that map to this code action.
+    fn to_kind(self) -> CodeActionKind {
+        match self {
+            Self::QuickFix => CodeActionKind::QUICKFIX,
+            Self::SourceFixAll => crate::SOURCE_FIX_ALL_RUFF,
+            Self::SourceOrganizeImports => crate::SOURCE_ORGANIZE_IMPORTS_RUFF,
+        }
+    }
+
+    fn from_kind(kind: CodeActionKind) -> impl Iterator<Item = Self> {
+        Self::all().filter(move |supported_kind| {
+            supported_kind.to_kind().as_str().starts_with(kind.as_str())
+        })
+    }
+
+    /// Returns all code actions kinds that the server currently supports.
+    fn all() -> impl Iterator<Item = Self> {
+        [
+            Self::QuickFix,
+            Self::SourceFixAll,
+            Self::SourceOrganizeImports,
+        ]
+        .into_iter()
+    }
+}
